import Flutter
import UIKit
import IswMobileSdkKit

public class SwiftIswMobileSdkPlugin: NSObject, FlutterPlugin {
    
    private var controller: UIViewController!
    
    init(controller: UIViewController) {
        self.controller = controller
    }
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channelId = "com.interswitchng.isw_mobile_sdk"
        let channel = FlutterMethodChannel(name: channelId, binaryMessenger: registrar.messenger())
        let controller = UIApplication.shared.delegate!.window!!.rootViewController!
        let instance = SwiftIswMobileSdkPlugin(controller: controller)
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        // get the arguments map
        guard let arguments = call.arguments as? [String: Any] else {
            return result(nil)
        }
        
        switch (call.method) {
        case "initialize":
            // retrieve the configuration and environment
            let configMapOrNull = arguments["config"] as? [String: String]
            let envNameOrNull = arguments["env"] as? String
            
            // ensure the values are not null
            guard let configMap = configMapOrNull, let envName = envNameOrNull else { return result(nil) }
            
            // get unwrapped configuration
            let config = getConfig(configMap: configMap)
            
            // initialize the sdk
            IswMobileSdk.intialize(config: config, env: getEnv(envName: envName))
            
            // return nil result
            result(nil);
            break;
            
        case "pay":
            guard let infoMap = arguments["paymentInfo"] as? [String: String] else { return }
            // get payment info from the arguments map
            let paymentInfo = getPaymentInfo(infoMap: infoMap)
            // create payment callback
            let callback = PaymentDelegate.init(result: result)
            // trigger payment with rootViewController and payment info
            IswMobileSdk.pay(on: self.controller, with: paymentInfo, call: callback)
            break;
        
        default: result(FlutterMethodNotImplemented);
        }
    }
}


extension SwiftIswMobileSdkPlugin {
    private func getConfig(configMap: [String: String]) -> IswSdkConfig {
        
        let merchantId = configMap["merchantId"]!,
                merchantSecret = configMap["merchantSecret"]!,
                merchantCode = configMap["merchantCode"]!,
                currencyCode = configMap["currencyCode"]!
        
        
        // create configuration
        let config = IswSdkConfig.init(
            clientId: merchantId,
            clientSecret: merchantSecret,
            currencyCode: currencyCode,
            merchantCode: merchantCode)
        
        return config
    }
    
    private func getEnv(envName: String) -> Environment {
        let env: Environment
        
        switch envName {
        case "PRODUCTION": env = .production
        case "SANDBOX": env = .sandbox
        default: env = .test
        }
        
        return env
    }
    
    private func getPaymentInfo(infoMap: [String: String]) -> IswPaymentInfo {
        
        let customerId = infoMap["customerId"]!,
                customerName = infoMap["customerName"]!,
                customerEmail = infoMap["customerEmail"]!,
                customerMobile = infoMap["customerMobile"]!,
                reference = infoMap["reference"]!,
                amount = infoMap["amount"]!
        
        // get the amount as an integer
        let amountAsInt = Int(amount)!
        
        return IswPaymentInfo.init(
            customerId: customerId,
            customerName: customerName,
            customerEmail: customerEmail,
            customerMobile: customerMobile,
            reference: reference,
            amount: amountAsInt)
    }
}

private class PaymentDelegate: IswPaymentDelegate {
    
    private var result: FlutterResult!
    
    init(result: @escaping FlutterResult) {
        self.result = result
    }
    
    public func onUserDidCancel() {
        guard let result = self.result else { return }
        // return nil as result
        result(nil)
    }
    
    public func onUserDidCompletePayment(result: IswPaymentResult) {
        guard let res = self.result else { return }
        // tranform the result into a map
        let resultMap = getResultMap(result: result)
        // return the payment result
        res(resultMap)
    }
    
    private func getResultMap(result: IswPaymentResult) -> [String: String] {
           let map: [String: String] = [
               "responseCode": result.responseCode,
               "responseDescription": result.responseDescription,
               "transactionReference": result.transactionReference,
               "amount": "\(result.amount)",
               "isSuccessful": "\(result.isSuccessful)",
               "channel": "\(result.channel)",
           ]
           
           return map
       }
       
    
}
